# Supplement: R script for model fitting

# W. C. Wetzel. 2014. Density-dependent recruitment structures a heterogeneous distribution 
# of herbivores among host plants. Ecology.

# install.packages('bbmle') # uncomment to install bbmle
library(bbmle)

data = read.csv('Wetzel 2014 Ecology - gall data.csv') # see Ecological Archives
# g11 is the number of galls on a plant before the experiment in 2011 (experimentally removed)
# f11 is the number of females added to each plant after gall removal in 2011
# g12 is the number of galls produced on each plant in 2012

# r is the intrinsic rate of increase
# K is the carrying capacity
# theta is the shape of density dependence parameter
# a and b determine the relationship between prior abundance and each of the above parameters

# To get initial values:
# make best guesses,
# run with simulated annealing (method='SANN'),
# enter MLEs from SANN as new starting values,
# and then run with Nelder-Mead (method='Nelder-Mead').
# For SANN increase the number of iterations to 500000 by
# adding control=list(maxit=500000)

# 1. Intercept model, constant per capita recruitment
m0 = mle2(g12 ~ dnbinom(mu = exp(r) * f11, size=size), 
	start=list(r=1.44, size=0.6424), data=data, method='Nelder-Mead')

# 2. No density-dependence, Effect of prior galls on r
mg = mle2(g12 ~ dnbinom(mu = exp(r + b * g11) * f11, size=size), 
	start=list(r=0.849, b=0.053, size=0.736), data=data, 
	method='Nelder-Mead')

# 3. plain theta-Ricker, no effect of prior galls
tf = mle2(g12 ~ dnbinom(mu = exp(r * (1 - (f11 / K)^theta)) * f11, 
	size=size), start=list(r=2.37, K=20.93, theta=0.56, size=0.722), 
	data=data, method='Nelder-Mead')

# 4. theta-Ricker with K = f(g11)
tfgk = mle2(g12 ~ dnbinom(mu = exp(r * (1 - (f11 / (a + b * g11))^theta)) * 
	f11, size=size), start=list(r=2.28, a=3.24, b=4.63, size=1.02, 
	theta=0.542), data=data, method='Nelder-Mead')

# 5. theta-Ricker with r = f(g11)
tfgr = mle2(g12 ~ dnbinom(mu = exp((a + b * g11) * (1 - (f11 / K)^theta)) * 
	f11, size=size), start=list(a=1.78, K=23.77, b=0.097, size=0.845, 
	theta=0.38), data=data, method='Nelder-Mead')

# 6. theta-Ricker with theta = f(g11)
tfgtheta = mle2(g12 ~ dnbinom(mu = exp(r * (1 - (f11 / K)^(a + b * g11)) ) * 
	f11, size=size), data=data, start=list(r=2.5, K=18.35, a=0.18, b=0.046, 
	size=.9), method='Nelder-Mead')

# AICc table with weights
AICctab(m0, mg, tf, tfgk, tfgr, tfgtheta, nobs=60, weights=TRUE)


# Brute-force check that the MLEs are the global optimum
# start searches at many different initial values

# Warning: this is slow with high 'reps'

reps = 2 # number of initial values for each parameter
# reps^4 different combinations of initial conditions

# build a data.frame in which to store results
# put in a range of parameter starting values
runs = expand.grid( rs = seq(0.1, 50, length=reps), 
		as = seq(0.1, 100, length=reps),
		bs = seq(0.1, 50, length=reps),
		sizes = 1,
		thetas = seq(0.01, 10, length=reps)
		)

# columns to store estimates and deviance
runs$r = NA
runs$a = NA
runs$b = NA
runs$size = NA
runs$theta = NA
runs$dev = NA


# loop through starting values
# try(..., silent=TRUE) silences errors so that the loop keeps running
# errors and non-convergence are stored as NA

for(i in 1:nrow(runs)) {
	tm =  try( mle2(g12 ~ dnbinom(mu = exp(r * (1 - (f11 / (a + b *
		 g11))^theta)) * f11, size=size), start=list(
			r=runs$rs[i], a=runs$as[i], b=runs$bs[i], 
			size=runs$sizes[i], theta=runs$thetas[i]), 
		data=data, method='SANN', control=list(maxit=20000), 
		skip.hessian=TRUE), silent=TRUE )
	if(class(tm) == 'try-error') {converg = 1} else {
		converg = tm@details$convergence}
	if(class(tm) == 'try-error' | converg > 0) {
		runs[i, c('r', 'a', 'b', 'size', 'theta', 'dev')] = NA} else {
	runs[i, c('r', 'a', 'b', 'size', 'theta')] = coef(tm)
	runs[i, 'dev'] = deviance(tm)
	}
}

# save a csv of runs
write.csv(runs, file='runs.csv')

# now restart each run at its MLEs and use Nelder-Mead

# get rid of NAs
runsc = runs[complete.cases(runs), ]

# make new columns for MLEs from Nelder-Mead
runsc$r2 = NA
runsc$a2 = NA
runsc$b2 = NA
runsc$size2 = NA
runsc$theta2 = NA
runsc$dev2 = NA

for(i in 1:nrow(runsc)) {
	tm =  try( mle2(g12 ~ dnbinom(mu = exp(r * (1 - (f11 / (a + b *
		g11))^theta)) * f11, size=size), start=list(
			r=runsc$r[i], a=runsc$a[i], b=runsc$b[i], 
			size=runsc$size[i], theta=runsc$theta[i]), 
		data=data, method='Nelder-Mead', skip.hessian=TRUE),
		silent=TRUE )
	if(class(tm) == 'try-error') {converg = 1} else {
		converg = tm@details$convergence}
	if(class(tm) == 'try-error' | converg > 0) {
		runsc[i, c('r2', 'a2', 'b2', 'size2', 'theta2', 'dev2')] = NA
		} else {
	runsc[i, c('r2', 'a2', 'b2', 'size2', 'theta2')] = coef(tm)
	runsc[i, 'dev2'] = deviance(tm)
	}
}


# results are in runsc
min(runsc$dev2, na.rm=TRUE)
deviance(tfgk)

# estimates with the lowest deviance
runsc[runsc$dev2 == min(runsc$dev2, na.rm=TRUE) & complete.cases(runsc), ]




